@@ -56,13 +56,20 @@ class _cb_container(object):
             else:
                 event = self._event
 
-            for m in [*self._m.parent._children, self._m.parent]:
-                # don't use "is" in here since Maps-children are proxies
-                # (and so are their attributes)!
-                if event.inaxes == m.figure.ax:
+            if self._method in ["keypress"]:
+                for m in [*self._m.parent._children, self._m.parent]:
+                    # always execute keypress callbacks irrespective of the mouse-pos
                     obj = self._getobj(m)
                     if obj is not None:
                         objs.append(obj)
+            else:
+                for m in [*self._m.parent._children, self._m.parent]:
+                    # don't use "is" in here since Maps-children are proxies
+                    # (and so are their attributes)!
+                    if event.inaxes == m.ax:
+                        obj = self._getobj(m)
+                        if obj is not None:
+                            objs.append(obj)
         return objs
 
     def _clear_temporary_artists(self):
@@ -116,6 +123,9 @@ class _cb_container(object):
                 if m1 is not m2:
                     self._getobj(m1)._fwd_cbs[id(m2)] = m2
 
+        if self._method == "click":
+            self._m.cb._click_move.share_events(*args)
+
     def add_temporary_artist(self, artist):
         """
         Make an artist temporary (remove it from the map at the next event)
@@ -128,6 +138,41 @@ class _cb_container(object):
         self._m.BM.add_artist(artist)
         self._temporary_artists.append(artist)
 
+    def _execute_cb(self, layer):
+        """
+        Get bool if a callback assigned on "layer" should be executed
+
+        - True if the callback is assigned to the "all" layer
+        - True if the corresponding layer is currently active
+        - True if the corresponding layer is part of a currently active "multi-layer"
+          (e.g.  "layer|layer2" )
+
+        Parameters
+        ----------
+        layer : str
+            The name of the layer to which the callback is attached.
+
+        Returns
+        -------
+        bool
+            Indicator if the callback should be executed on the currently visible
+            layer or not.
+        """
+        visible_layer = self._m.BM.bg_layer
+
+        if layer == "all":
+            # the all layer is always executed
+            return True
+        elif "|" in visible_layer:
+            if layer == visible_layer:
+                # return true for the multi-layer itself
+                return True
+            else:
+                # return true for layers that are part of the multi-layer
+                return any(i.strip() == layer for i in visible_layer.split("|"))
+        else:
+            return layer == visible_layer
+
 
 class _click_container(_cb_container):
     """
@@ -190,8 +235,8 @@ class _click_container(_cb_container):
 
         For additional keyword-arguments check the doc of the callback-functions!
 
-        Examples:
-        ---------
+        Examples
+        --------
         Get a (temporary) annotation on a LEFT-double-click:
 
             >>> m.cb.click.attach.annotate(double_click=True, button=1, permanent=False)
@@ -289,7 +334,7 @@ class _click_container(_cb_container):
 
             if self._parent._method == "pick":
                 assert (
-                    self._parent._m.figure.coll is not None
+                    self._parent._m.coll is not None
                 ), "you can only attach pick-callbacks after calling `plot_map()`!"
 
             return self._parent._add_callback(
@@ -351,6 +396,35 @@ class _click_container(_cb_container):
 
             return cbs
 
+    def _parse_cid(self, cid):
+        """
+        get the
+
+        Parameters
+        ----------
+        cid : TYPE
+            DESCRIPTION.
+
+        Returns
+        -------
+        name : str
+            the callback name.
+        layer : str
+            the layer to which the callback is attached.
+        ds : str
+            indicator if double- or single-click is used.
+        b : str
+            the button (e.g. 1, 2, 3 for left, middle, right)
+        m : str
+            the keypress modifier.
+        """
+        # do this to allow double-underscores in the layer-name
+
+        name, rest = cid.split("__", 1)
+        layer, ds, b, m = rest.rsplit("__", 3)
+
+        return name, layer, ds, b, m
+
     def remove(self, callback=None):
         """
         Remove previously attached callbacks from the map.
@@ -367,10 +441,10 @@ class _click_container(_cb_container):
             if callback in self._connected_move_cbs:
                 for i in self._connected_move_cbs[callback]:
                     self._m.cb._click_move.remove(i)
+                self._connected_move_cbs.pop(callback)
 
         if callback is not None:
-            s = callback.split("__")
-            name, layer, ds, b, m = s
+            name, layer, ds, b, m = self._parse_cid(callback)
 
         cbname = name + "__" + layer
         bname = f"{b}__{m}"
@@ -519,7 +593,7 @@ class _click_container(_cb_container):
             button = self._default_button
 
         if self._method == "pick":
-            assert self._m.figure.coll is not None, (
+            assert self._m.coll is not None, (
                 "you can only attach pick-callbacks after plotting a dataset!"
                 + "... use `m.plot_map()` first."
             )
@@ -567,7 +641,7 @@ class _click_container(_cb_container):
         # get a unique name for the callback
         # name_idx__layer
         ncb = [
-            int(i.split("__", 1)[0].rsplit("_", 1)[1])
+            int(i.split("__")[0].rsplit("_", 1)[1])
             for i in d
             if i.startswith(callback.__name__)
         ]
@@ -661,8 +735,8 @@ class cb_click_container(_click_container):
             bcbs = cbs[button_modifier]
 
             for key in self._sort_cbs(bcbs):
-                layer = key.split("__")[1]
-                if layer != "all" and layer != str(self._m.BM.bg_layer):
+                layer = key.split("__", 1)[1]
+                if not self._execute_cb(layer):
                     return
 
                 cb = bcbs[key]
@@ -692,15 +766,15 @@ class cb_click_container(_click_container):
 
     def _reset_cids(self):
         if self._cid_button_press_event:
-            self._m.figure.f.canvas.mpl_disconnect(self._cid_button_press_event)
+            self._m.f.canvas.mpl_disconnect(self._cid_button_press_event)
         self._cid_button_press_event = None
 
         if self._cid_motion_event:
-            self._m.figure.f.canvas.mpl_disconnect(self._cid_motion_event)
+            self._m.f.canvas.mpl_disconnect(self._cid_motion_event)
         self._cid_motion_event = None
 
         if self._cid_button_release_event:
-            self._m.figure.f.canvas.mpl_disconnect(self._cid_button_release_event)
+            self._m.f.canvas.mpl_disconnect(self._cid_button_release_event)
         self._cid_button_release_event = None
 
     def _add_click_callback(self):
@@ -713,23 +787,23 @@ class cb_click_container(_click_container):
                     return
                 # don't execute callbacks if a toolbar-action is active
                 if (
-                    self._m.figure.f.canvas.toolbar is not None
-                ) and self._m.figure.f.canvas.toolbar.mode != "":
+                    self._m.f.canvas.toolbar is not None
+                ) and self._m.f.canvas.toolbar.mode != "":
                     return
 
                 # execute onclick on the maps object that belongs to the clicked axis
                 # and forward the event to all forwarded maps-objects
-
                 for obj in self._objs:
                     # clear temporary artists before executing new callbacks to avoid
                     # having old artists around when callbacks are triggered again
                     obj._clear_temporary_artists()
-                    self._m.BM._clear_temp_artists(self._method)
                     obj._onclick(event)
 
                     # forward callbacks to the connected maps-objects
                     obj._fwd_cb(event)
 
+                self._m.BM._clear_temp_artists(self._method)
+
                 self._m.parent.BM.update(clear=self._method)
             except ReferenceError:
                 pass
@@ -741,13 +815,10 @@ class cb_click_container(_click_container):
                     return
                 # don't execute callbacks if a toolbar-action is active
                 if (
-                    self._m.figure.f.canvas.toolbar is not None
-                ) and self._m.figure.f.canvas.toolbar.mode != "":
+                    self._m.f.canvas.toolbar is not None
+                ) and self._m.f.canvas.toolbar.mode != "":
                     return
 
-                # clear temporary click artists when the mouse-button is released
-                self._m.BM._clear_temp_artists(self._method)
-
                 # execute onclick on the maps object that belongs to the clicked axis
                 # and forward the event to all forwarded maps-objects
                 for obj in self._objs:
@@ -761,23 +832,22 @@ class cb_click_container(_click_container):
             except ReferenceError:
                 # ignore errors caused by no-longer existing weakrefs
                 pass
-            # self._m.parent.BM.update(clear=False)
 
         if self._cid_button_press_event is None:
             # ------------- add a callback
-            self._cid_button_press_event = self._m.figure.f.canvas.mpl_connect(
+            self._cid_button_press_event = self._m.f.canvas.mpl_connect(
                 "button_press_event", clickcb
             )
 
         if self._cid_button_release_event is None:
             # ------------- add a callback
-            self._cid_button_release_event = self._m.figure.f.canvas.mpl_connect(
+            self._cid_button_release_event = self._m.f.canvas.mpl_connect(
                 "button_release_event", releasecb
             )
 
     def _fwd_cb(self, event):
         # click container events are MouseEvents!
-        if event.inaxes != self._m.figure.ax:
+        if event.inaxes != self._m.ax:
             return
 
         if event.name == "button_release_event":
@@ -790,6 +860,8 @@ class cb_click_container(_click_container):
         else:
             for key, m in self._fwd_cbs.items():
                 obj = self._getobj(m)
+                # clear all temporary artists that are still around
+                obj._clear_temporary_artists()
                 if obj is None:
                     continue
 
@@ -803,7 +875,7 @@ class cb_click_container(_click_container):
                 xdata, ydata = transformer.transform(event.xdata, event.ydata)
 
                 dummymouseevent = SimpleNamespace(
-                    inaxes=m.figure.ax,
+                    inaxes=m.ax,
                     dblclick=event.dblclick,
                     button=event.button,
                     xdata=xdata,
@@ -814,9 +886,6 @@ class cb_click_container(_click_container):
                 )
 
                 obj._onclick(dummymouseevent)
-                # append clear-action again since it will already be executed
-                # by the first click!
-                m.BM._after_update_actions.append(obj._clear_temporary_artists)
 
 
 class cb_move_container(cb_click_container):
@@ -857,7 +926,7 @@ class cb_move_container(cb_click_container):
 
     def _reset_cids(self):
         if self._cid_motion_event:
-            self._m.figure.f.canvas.mpl_disconnect(self._cid_motion_event)
+            self._m.f.canvas.mpl_disconnect(self._cid_motion_event)
         self._cid_motion_event = None
 
     def _add_move_callback(self):
@@ -867,7 +936,7 @@ class cb_move_container(cb_click_container):
                 # only execute movecb if a mouse-button is holded down
                 # and only if the motion is happening inside the axes
                 if self._button_down:
-                    if not event.button:  # or (event.inaxes != self._m.figure.ax):
+                    if not event.button:  # or (event.inaxes != self._m.ax):
                         # always clear temporary move-artists
                         if self._method == "move":
                             for obj in self._objs:
@@ -875,7 +944,7 @@ class cb_move_container(cb_click_container):
                             self._m.BM._clear_temp_artists(self._method)
                         return
                 else:
-                    if event.button:  # or (event.inaxes != self._m.figure.ax):
+                    if event.button:  # or (event.inaxes != self._m.ax):
                         # always clear temporary move-artists
                         if self._method == "move":
                             for obj in self._objs:
@@ -888,8 +957,8 @@ class cb_move_container(cb_click_container):
                     return
                 # don't execute callbacks if a toolbar-action is active
                 if (
-                    self._m.figure.f.canvas.toolbar is not None
-                ) and self._m.figure.f.canvas.toolbar.mode != "":
+                    self._m.f.canvas.toolbar is not None
+                ) and self._m.f.canvas.toolbar.mode != "":
                     return
 
                 # execute onclick on the maps object that belongs to the clicked axis
@@ -910,7 +979,7 @@ class cb_move_container(cb_click_container):
 
         if self._cid_motion_event is None:
             # for click-callbacks, allow motion-detection
-            self._cid_motion_event = self._m.figure.f.canvas.mpl_connect(
+            self._cid_motion_event = self._m.f.canvas.mpl_connect(
                 "motion_notify_event", movecb
             )
 
@@ -982,9 +1051,8 @@ class cb_pick_container(_click_container):
         self._add_pick_callback()
 
     def _default_picker(self, artist, event):
-
         # make sure that objects are only picked if we are on the right layer
-        if self._m.layer != "all" and self._m.layer != self._m.BM.bg_layer:
+        if not self._execute_cb(self._m.layer):
             return False, None
 
         try:
@@ -1019,17 +1087,40 @@ class cb_pick_container(_click_container):
 
         return False, None
 
+    def _get_id(self, ind):
+        """
+        Identify the ID from a 1D list or range object or a numpy.ndarray
+        (to avoid very large numpy-arrays if no explicit IDs are provided)
+
+        Parameters
+        ----------
+        ind : int
+            The index of the flattened array.
+
+        Returns
+        -------
+        ID : any
+            The corresponding data-ID.
+        """
+
+        ids = self._m._props["ids"]
+        if isinstance(ids, (list, range)):
+            ID = ids[ind]
+        elif isinstance(ids, np.ndarray):
+            ID = ids.flat[ind]
+        else:
+            ID = "?"
+
+        return ID
+
     def _get_pickdict(self, event):
         ind = event.ind
         if ind is not None:
-            if self._m.figure.coll is not None and event.artist is self._m.figure.coll:
+            if self._m.coll is not None and event.artist is self._m.coll:
                 clickdict = dict(
-                    pos=(
-                        self._m._props["x0"].flat[ind],
-                        self._m._props["y0"].flat[ind],
-                    ),
-                    ID=self._m._props["ids"].flat[ind],
-                    val=self._m._props["z_data"][ind],
+                    pos=self._m._get_xy_from_index(ind, reprojected=True),
+                    ID=self._get_id(ind),
+                    val=self._m._props["z_data"].flat[ind],
                     ind=ind,
                     picker_name=self._picker_name,
                 )
@@ -1051,13 +1142,13 @@ class cb_pick_container(_click_container):
 
         # only execute onpick if the correct layer is visible
         # (relevant for forwarded callbacks)
-        if self._m.layer != "all" and self._m.layer != self._m.BM.bg_layer:
+        if not self._execute_cb(self._m.layer):
             return
 
         # don't execute callbacks if a toolbar-action is active
         if (
-            self._m.figure.f.canvas.toolbar is not None
-        ) and self._m.figure.f.canvas.toolbar.mode != "":
+            self._m.f.canvas.toolbar is not None
+        ) and self._m.f.canvas.toolbar.mode != "":
             return
 
         clickdict = self._get_pickdict(event)
@@ -1083,9 +1174,8 @@ class cb_pick_container(_click_container):
             bcbs = cbs[button_modifier]
 
             for key in self._sort_cbs(bcbs):
-                layer = key.split("__")[1]
-                if layer != "all" and layer != str(self._m.BM.bg_layer):
-                    # TODO
+                layer = key.split("__", 1)[1]
+                if not self._execute_cb(layer):
                     # only execute callbacks if the layer name of the associated
                     # maps-object is active
                     return
@@ -1096,17 +1186,17 @@ class cb_pick_container(_click_container):
 
     def _reset_cids(self):
         for method, cid in self._cid_pick_event.items():
-            self._m.figure.f.canvas.mpl_disconnect(cid)
+            self._m.f.canvas.mpl_disconnect(cid)
         self._cid_pick_event.clear()
 
     def _add_pick_callback(self):
         # execute onpick and forward the event to all connected Maps-objects
 
         def pickcb(event):
-            try:
 
+            try:
                 # make sure pickcb is only executed if we are on the right layer
-                if self._m.layer != "all" and self._m.layer != self._m.BM.bg_layer:
+                if not self._execute_cb(self._m.layer):
                     return
 
                 # check if we want to ignore callbacks
@@ -1115,8 +1205,8 @@ class cb_pick_container(_click_container):
 
                 # don't execute callbacks if a toolbar-action is active
                 if (
-                    self._m.figure.f.canvas.toolbar is not None
-                ) and self._m.figure.f.canvas.toolbar.mode != "":
+                    self._m.f.canvas.toolbar is not None
+                ) and self._m.f.canvas.toolbar.mode != "":
                     return
 
                 if not self._artist is event.artist:
@@ -1148,17 +1238,18 @@ class cb_pick_container(_click_container):
 
         # attach the callbacks (only once per method!)
         if self._method not in self._cid_pick_event:
-            self._cid_pick_event[self._method] = self._m.figure.f.canvas.mpl_connect(
+            self._cid_pick_event[self._method] = self._m.f.canvas.mpl_connect(
                 "pick_event", pickcb
             )
 
     def _fwd_cb(self, event, picker_name):
         # PickEvents have a .mouseevent property for the associated MouseEvent!
-        if event.mouseevent.inaxes != self._m.figure.ax:
+        if event.mouseevent.inaxes != self._m.ax:
             return
 
         for key, m in self._fwd_cbs.items():
             obj = self._getobj(m)
+            obj._clear_temporary_artists()
             if obj is None:
                 continue
 
@@ -1175,7 +1266,7 @@ class cb_pick_container(_click_container):
             )
 
             dummymouseevent = SimpleNamespace(
-                inaxes=m.figure.ax,
+                inaxes=m.ax,
                 dblclick=event.mouseevent.dblclick,
                 button=event.mouseevent.button,
                 xdata=xdata,
@@ -1188,7 +1279,7 @@ class cb_pick_container(_click_container):
                 artist=obj._artist,
                 dblclick=event.mouseevent.dblclick,
                 button=event.mouseevent.button,
-                # inaxes=m.figure.ax,
+                # inaxes=m.ax,
                 mouseevent=dummymouseevent,
                 # picker_name=picker_name,
             )
@@ -1207,7 +1298,6 @@ class cb_pick_container(_click_container):
                 dummyevent.dist = None
 
             obj._onpick(dummyevent)
-            m.BM._after_update_actions.append(obj._clear_temporary_artists)
 
 
 class keypress_container(_cb_container):
@@ -1247,7 +1337,7 @@ class keypress_container(_cb_container):
 
     def _reset_cids(self):
         if self._cid_keypress_event:
-            self._m.figure.f.canvas.mpl_disconnect(self._cid_keypress_event)
+            self._m.f.canvas.mpl_disconnect(self._cid_keypress_event)
         self._cid_keypress_event = None
 
     def _initialize_callbacks(self):
@@ -1285,26 +1375,25 @@ class keypress_container(_cb_container):
                 update = False
                 for obj in self._objs:
                     # only trigger callbacks on the right layer
-                    if (obj._m.layer != "all") and (
-                        obj._m.layer != self._m.BM.bg_layer
-                    ):
+                    if not self._execute_cb(obj._m.layer):
                         continue
-                    if event.key in obj.get.cbs:
+                    if any(i in obj.get.cbs for i in (event.key, None)):
                         update = True
                         # do this to allow deleting callbacks with a callback
                         # otherwise modifying a dict during iteration is problematic!
-                        cbs = obj.get.cbs[event.key]
+                        cbs = {**obj.get.cbs[event.key], **obj.get.cbs[None]}
+
                         names = list(cbs)
                         for name in names:
                             if name in cbs:
                                 cbs[name](key=event.key)
-                if update:
-                    self._m.parent.BM.update(clear=self._method)
+                # if update:
+                #     self._m.parent.BM.update(clear=self._method)
             except ReferenceError:
                 pass
 
         if self._m is self._m.parent:
-            self._cid_keypress_event = self._m.figure.f.canvas.mpl_connect(
+            self._cid_keypress_event = self._m.f.canvas.mpl_connect(
                 "key_press_event", _onpress
             )
 
@@ -1314,17 +1403,28 @@ class keypress_container(_cb_container):
 
         Each callback takes 1 additional keyword-arguments:
 
-        key : str
-            the key to use
-            (modifiers are attached with a '+', e.g. "alt+d" )
+        key : str or None
+            The key to use.
+
+            - Modifiers are attached with a '+', e.g. "alt+d"
+            - If None, the callback will be fired on any key!
 
         For additional keyword-arguments check the doc of the callback-functions!
 
         Examples
         --------
 
+            Attach a pre-defined callback:
+
             >>> m.cb.keypress.attach.switch_layer(layer=1, key="1")
 
+            Attach a custom callback:
+
+            >>> def cb(**kwargs):
+            >>>     ... do something ...
+            >>>
+            >>> m.cb.keypress.attach(cb, key="3")
+
         """
 
         def __init__(self, parent):
@@ -1343,7 +1443,7 @@ class keypress_container(_cb_container):
 
         def __call__(self, f, key, **kwargs):
             """
-            add a custom callback-function to the map
+            Add a custom callback-function to the map
 
             Parameters
             ----------
@@ -1355,10 +1455,11 @@ class keypress_container(_cb_container):
                 >>>     print("hello world, asdf=", asdf)
                 >>>
                 >>> m.cb.attach(some_callback, asdf=1)
+            key : str or None
+                The key to use.
 
-            key : str
-                the key to use
-                (modifiers are attached with a '+', e.g. "alt+d" )
+                - Modifiers are attached with a '+', e.g. "alt+d"
+                - If None, the callback will be fired on any key!
 
             **kwargs :
                 kwargs passed to the callback-function
@@ -1371,7 +1472,7 @@ class keypress_container(_cb_container):
 
             """
 
-            if not isinstance(key, str):
+            if key is not None and not isinstance(key, str):
                 raise TypeError(
                     "EOmaps: The 'key' for keypress-callbacks must be a string!"
                 )
@@ -1394,6 +1495,12 @@ class keypress_container(_cb_container):
 
             return cbs
 
+    def _parse_cid(self, cid):
+        name, rest = cid.split("__", 1)
+        layer, key = rest.rsplit("__", 1)
+
+        return name, layer, key
+
     def remove(self, callback=None):
         """
         remove an attached callback from the figure
@@ -1406,7 +1513,7 @@ class keypress_container(_cb_container):
         """
 
         if callback is not None:
-            name, layer, key = callback.split("__")
+            name, layer, key = self._parse_cid(callback)
 
         cbname = name + "__" + layer
 
@@ -1442,9 +1549,11 @@ class keypress_container(_cb_container):
         callback : callable or str
             The callback-function to attach.
 
-        key : str
-            the key to use
-            (modifiers are attached with a '+', e.g. "alt+d" )
+        key : str or None
+            The key to use.
+
+            - Modifiers are attached with a '+', e.g. "alt+d"
+            - If None, the callback will be fired on any key!
 
         **kwargs :
             kwargs passed to the callback-function
@@ -1467,9 +1576,12 @@ class keypress_container(_cb_container):
             callback = getattr(self._cb, callback)
 
         cbdict = self.get.cbs[key]
+
         # get a unique name for the callback
         ncb = [
-            int(i.rsplit("_", 1)[1]) for i in cbdict if i.startswith(callback.__name__)
+            int(i.rsplit("__", 1)[0].rsplit("_", 1)[1])
+            for i in cbdict
+            if i.startswith(callback.__name__)
         ]
         cbkey = (
             callback.__name__
