@@ -90,151 +90,83 @@ class map_objects(object):
     ):
         self._m = m
 
-        self.coll = None  # self.coll is assigned in "m.plot_map()"
+        self._coll = None  # self.coll is assigned in "m.plot_map()"
         self._figure_closed = False
 
+    @property
+    def coll(self):
+        return self._coll
+
+    @coll.setter
+    def coll(self, val):
+        self._coll = val
+
+    @coll.getter
+    def coll(self):
+        warn(
+            "EOmaps: Using `m.figure.coll` is depreciated in EOmaps v5.x."
+            "Use `m.coll` instead!"
+        )
+        return self._coll
+
     @property
     def f(self):
+        warn(
+            "EOmaps: Using `m.figure.f` is depreciated in EOmaps v5.x."
+            "Use `m.f` instead!"
+        )
         # always return the figure of the parent object
-        return self._m.parent._f
+        return self._m.f
 
     @property
     def ax(self):
-        ax = self._m._ax
-
-        # return None in case the plot is not yet initialized
-        if isinstance(ax, SubplotSpec):
-            ax = None
+        warn(
+            "EOmaps: Using `m.figure.ax` is depreciated in EOmaps v5.x."
+            "Use `m.ax` instead!"
+        )
+        ax = self._m.ax
         return ax
 
-    @property
-    def ax_cb(self):
-        return getattr(self._m, "_ax_cb", None)
-
-    @property
-    def ax_cb_plot(self):
-        return getattr(self._m, "_ax_cb_plot", None)
-
     @property
     def gridspec(self):
+        warn(
+            "EOmaps: Using `m.figure.gridspec` is depreciated in EOmaps v5.x."
+            "Use `m._gridspec` instead!"
+        )
         return getattr(self._m, "_gridspec", None)
 
     @property
-    def cb_gridspec(self):
-        return getattr(self._m, "_cb_gridspec", None)
+    def ax_cb(self):
+        warn(
+            "EOmaps: Using `m.figure.ax_cb` is depreciated in EOmaps v5.x."
+            "Use `m.colorbar.ax_cb` instead!"
+        )
+        colorbar = getattr(self._m, "colorbar", None)
+        if colorbar is not None:
+            return colorbar.ax_cb
+
+    @property
+    def ax_cb_plot(self):
+        warn(
+            "EOmaps: Using `m.figure.ax_cb_plot` is depreciated in EOmaps v5.x."
+            "Use `m.colorbar.ax_cb_plot` instead!"
+        )
+        colorbar = getattr(self._m, "colorbar", None)
+        if colorbar is not None:
+            return colorbar.ax_cb_plot
 
-    # @wraps(plt.Axes.set_position)
     def set_colorbar_position(self, pos=None, ratio=None, cb=None):
         """
-        Set the position (and ratio) of a previously created colorbar with a histogram.
-
-        Parameters
-        ----------
-        pos : list    [left, bottom, width, height]
-            The bounding-box of the colorbar & histogram in relative
-            units [0,1] (with respect to the figure)
-            If None the current position is maintained.
-        ratio : float, optional
-            The ratio between the size of the colorbar and the size of the histogram.
-            'ratio=10' means that the histogram is 10 times as large as the colorbar!
-            The default is None in which case the current ratio is maintained.
-        cb : list, optional
-            The colorbar-objects (as returned by `m.add_colorbar()`)
-            If None, the existing colorbar will be used.
-
-        Examples
-        --------
-
-        Change both the position and the ratio between colorbar and histogram
-
-        >>> m.figure.set_colorbar_position(pos=[.1, .1, .8, .2], ratio=15)
-
-        Change only the ratio between colorbar and histogram
-
-        >>> m.figure.set_colorbar_position(ratio=999)
-
-        Change only the position of colorbar and histogram
-
-        >>> m.figure.set_colorbar_position(pos=[.1, .1, .8, .2])
+        This function is depreciated in EOmaps v5.x!
 
+        Use the following methods instead:
+        - m.colorbar.set_position
+        - m.colorbar.set_hist_size
         """
-
-        if cb is None:
-            (
-                _,
-                _,
-                ax_cb,
-                ax_cb_plot,
-                ax_cb_extend,
-                extend_frac,
-                orientation,
-                _,
-            ) = self._m._colorbar
-        else:
-            _, _, ax_cb, ax_cb_plot, ax_cb_extend, extend_frac, orientation, _ = cb
-
-        if orientation == "horizontal":
-            pcb = ax_cb.get_position()
-            pcbp = ax_cb_plot.get_position()
-
-            if pos is None:
-                pos = [pcb.x0, pcb.y0, pcb.width, pcb.height + pcbp.height]
-            if ratio is None:
-                ratio = pcbp.height / pcb.height
-
-            hcb = pos[3] / (1 + ratio)
-            hp = ratio * hcb
-
-            ax_cb.set_position(
-                [pos[0], pos[1], pos[2], hcb],
-            )
-            ax_cb_plot.set_position(
-                [pos[0], pos[1] + hcb, pos[2], hp],
-            )
-
-            # adjust colorbar extension arrows
-            if ax_cb_extend:
-                frac = (
-                    ax_cb.bbox.transformed(ax_cb.figure.transFigure.inverted()).width
-                    * extend_frac
-                )
-                ax_cb_extend.set_position(
-                    [pos[0] - frac / 2, pos[1], pos[2] + frac, hcb],
-                )
-
-        elif orientation == "vertical":
-            pcb = ax_cb.get_position()
-            pcbp = ax_cb_plot.get_position()
-            if pos is None:
-                pos = [pcbp.x0, pcbp.y0, pcb.width + pcbp.width, pcb.height]
-            if ratio is None:
-                ratio = pcbp.width / pcb.width
-
-            wcb = pos[2] / (1 + ratio)
-            wp = ratio * wcb
-
-            ax_cb.set_position(
-                [pos[0] + wp, pos[1], wcb, pos[3]],
-            )
-            ax_cb_plot.set_position(
-                [pos[0], pos[1], wp, pos[3]],
-            )
-
-            # adjust colorbar extension arrows
-            if ax_cb_extend:
-                frac = (
-                    ax_cb.bbox.transformed(ax_cb.figure.transFigure.inverted()).height
-                    * extend_frac
-                )
-                ax_cb_extend.set_position(
-                    [pos[0] + wp, pos[1] - frac / 2, wcb, pos[3] + frac],
-                )
-
-        else:
-            raise TypeError(f"EOmaps: '{orientation}' is not a valid orientation")
-
-        # re-fetch the background layer to make changes visible
-        self._m.BM.fetch_bg(self._m.layer)
+        raise AssertionError(
+            "EOmaps: `m.figure.set_colorbar_position` is depreciated in EOmaps v5.x! "
+            "use `m.colorbar.set_position` and `m.colorbar.set_hist_size` instead."
+        )
 
 
 class data_specs(object):
@@ -654,8 +586,8 @@ class _NaturalEarth_presets:
             self._m,
             "physical",
             "coastline",
-            fc="none",
-            ec="k",
+            facecolor="none",
+            edgecolor="k",
             zorder=100,
         )
 
@@ -676,7 +608,7 @@ class _NaturalEarth_presets:
         color = rgb2hex(cfeature.COLORS["water"])
 
         return self._feature(
-            self._m, "physical", "ocean", fc=color, ec="none", zorder=-1
+            self._m, "physical", "ocean", facecolor=color, edgecolor="none", zorder=-1
         )
 
     @property
@@ -697,7 +629,7 @@ class _NaturalEarth_presets:
         color = rgb2hex(cfeature.COLORS["land"])
 
         return self._feature(
-            self._m, "physical", "land", fc=color, ec="none", zorder=-1
+            self._m, "physical", "land", facecolor=color, edgecolor="none", zorder=-1
         )
 
     @property
@@ -718,9 +650,9 @@ class _NaturalEarth_presets:
             self._m,
             "cultural",
             "admin_0_countries",
-            fc="none",
-            ec=".5",
-            lw=0.5,
+            facecolor="none",
+            edgecolor=".5",
+            linewidth=0.5,
             zorder=99,
         )
 
@@ -751,13 +683,11 @@ class _NaturalEarth_presets:
         def __call__(self, scale="50m", **kwargs):
             k = dict(**self.kwargs)
             k.update(kwargs)
-
             self.feature = getattr(
                 getattr(self._m.add_feature, f"{self.category}_{scale}"), self.name
             )
 
             self.__doc__ = self.feature.__doc__
-
             return self.feature(**k)
 
 
@@ -1067,7 +997,7 @@ class NaturalEarth_features(object):
                     else:
                         uselayer = layer
                     self.feature._kwargs.update(kwargs)
-                    art = m.figure.ax.add_feature(self.feature)
+                    art = m.ax.add_feature(self.feature)
 
                     m.BM.add_bg_artist(art, layer=uselayer)
             else:
@@ -1471,7 +1401,6 @@ else:
 
                 - default: standard OSM layer
                 - default_german: standard OSM layer in german
-                - standard: standard OSM layer
                 - stamen_toner: Black and white style by stamen
                     - stamen_toner_lines
                     - stamen_toner_background
@@ -1483,8 +1412,11 @@ else:
                     - stamen_terrain_lines
                     - stamen_terrain_labels
                     - stamen_terrain_background
+
                 - OSM_terrestis: Styles hosted as free WMS service by Terrestis
                 - OSM_mundialis: Styles hosted as free WMS service by Mundialis
+                - OSM_wms and OSM_landuse: WMS hosted by Heidelberg Institute for
+                  Geoinformation Technology
 
             Note
             ----
@@ -1497,6 +1429,7 @@ else:
 
             - for OSM_terrestis: https://www.terrestris.de/en/openstreetmap-wms/
             - for OSM_mundialis: https://www.mundialis.de/en/ows-mundialis/
+            - for OSM_wms and OSM_landuse : https://heigit.org
 
             """
 
@@ -1737,6 +1670,103 @@ else:
                 )
                 return WMS
 
+            @property
+            @lru_cache()
+            def OSM_wms(self):
+                WMS = _WebServiec_collection(
+                    m=self._m,
+                    service_type="wms",
+                    url=r"https://maps.heigit.org/osm-wms/service?REQUEST=GetCapabilities&SERVICE=WMS",
+                )
+                WMS.__doc__ = combdoc(
+                    type(self).__doc__,
+                    """
+                    The first version of osm-wms.de was put online at 13th of February
+                    2009. Since these days it serves OpenStreetMap based maps. As the
+                    name indicates it does provide the maps via the OGC-WMS format
+                    other than the usual map tile providers. This increases the
+                    flexibility of the usage of this service as you are able to use the
+                    map not only by using this website, but also by registering the wms
+                    service in any GIS software that supports the standard protocol of
+                    the OpenGeospatialConsortium OGC-WMS.
+
+                    Note
+                    ----
+                    **LICENSE-info (without any warranty for correctness!!)**
+
+                    Terms of Use
+                    The usage for non-commercial, scientific or personal use is free.
+                    If you use our services, please acknowledge and refer to our
+                    website: osm-wms.de
+
+                    Acknowledgements:
+
+                    This work has kindly been supported by the Klaus Tschira Foundation
+                    (KTS) Heidelberg in the context of establishing the Heidelberg
+                    Institute for Geoinformation Technology (HeiGIT).
+
+                    Attributions:
+
+                    OSM Data: Licensed under ODbL, © OpenStreetMap contributors
+                    Original Data for Hillshade: CIAT-CSI SRTM
+                    Terms of use from CIAT: Users are prohibited from any commercial,
+                    non-free resale, or redistribution without explicit written
+                    permission from CIAT. Original data by Jarvis A., H.I. Reuter,
+                    A. Nelson, E. Guevara, 2008, Hole-filled seamless SRTM data V4,
+                    International Centre for Tropical Agriculture (CIAT), available
+                    from https://srtm.csi.cgiar.org.
+
+                    For more details, please visit:
+                    https://osm-wms.de
+                    """,
+                )
+                return WMS
+
+            @property
+            @lru_cache()
+            def OSM_landuse(self):
+                WMS = _WebServiec_collection(
+                    m=self._m,
+                    service_type="wms",
+                    url=r"https://maps.heigit.org/osmlanduse/service?REQUEST=GetCapabilities",
+                )
+                WMS.__doc__ = combdoc(
+                    type(self).__doc__,
+                    """
+                    OSM Landuse Landcover is a WebGIS application to explore the
+                    OpenStreetMap database specifically in terms of landuse and
+                    landcover information. Land use tags were predicted when absent
+                    using belows (Schultz et al. 2020 in prep, Schultz et al. 2017)
+                    method. This was first addressed for Germany (2017) and now (2020)
+                    - with the improved methods - for all EU countries.
+
+                    Note
+                    ----
+                    **LICENSE-info (without any warranty for correctness!!)**
+
+                    Terms of use:
+
+                    Overlay tiles of "OSM Landuse Landcover" can be used freely and
+                    without charge by any individuals through this website.
+                    If you intend to use tiles or statistics from
+                    "OSM Landuse Landcover" services in your own applications please
+                    contact us.
+                    Commercial usage of the services provided by "OSM Landuse Landcover"
+                    does need approval!
+                    OpenStreetMap data is available under the Open Database License.
+
+                    Acknowledgements:
+
+                    This work is supported by Heidelberg Institute for Geoinformation
+                    Technology (https://heigit.org)
+
+
+                    For more details, plese visit:
+                    https://osmlanduse.org
+                    """,
+                )
+                return WMS
+
         @property
         @lru_cache()
         def EEA_DiscoMap(self):
@@ -2213,6 +2243,49 @@ else:
             WMS.__doc__ = type(self).CAMS.__doc__
             return WMS
 
+        @property
+        @lru_cache()
+        def DLR_basemaps(self):
+            """
+            A collection of basemaps provided by the EOC Geoservice of the Earth
+            Observation Center (EOC) of the German Aerospace Center (DLR).
+
+            https://geoservice.dlr.de
+
+            Note
+            ----
+            **LICENSE-info (without any warranty for correctness!!)**
+
+            The background maps and overlays used on this site are created and published
+            by DLR for informational and illustration purposes only. The user assumes
+            the entire risk related to the use of these data. These maps may contain
+            errors and therefore users of these maps should review or consult the
+            primary data and information sources to ascertain the usability of the
+            information.
+
+            Disclaimer
+
+            Although DLR is making these maps available to others, DLR does not warrant,
+            endorse, or recommend the use of these maps for any given purpose. DLR is
+            providing these data "as is", and disclaims any and all warranties, whether
+            expressed or implied. In no event will DLR be liable to you or to any third
+            party for any direct, indirect, incidental, consequential, special, or
+            exemplary damages or lost profits resulting from any use or misuse of
+            these data. The user of these maps cannot claim any rights pertaining to
+            its usage.
+
+            (check: https://geoservice.dlr.de/web/about for full details)
+            """
+
+            WMS = _WebServiec_collection(
+                m=self._m,
+                service_type="wms",
+                url="https://geoservice.dlr.de/eoc/basemap/wms?SERVICE=WMS&REQUEST=GetCapabilities",
+            )
+
+            WMS.__doc__ = type(self).CAMS.__doc__
+            return WMS
+
         @property
         @lru_cache()
         def ESRI_ArcGIS(self):
