@@ -71,6 +71,7 @@ class shapes(object):
         "geod_circles",
         "ellipses",
         "rectangles",
+        "raster",
         "voronoi_diagram",
         "delaunay_triangulation",
         "shade_points",
@@ -132,11 +133,22 @@ class shapes(object):
             )
 
         radius = None
-
         # try to estimate radius for 2D datasets
-        if isinstance(m.data_specs.x, np.ndarray) and len(m.data_specs.x.shape) == 2:
-            radiusx = np.nanmedian(np.diff(x.reshape(m.data_specs.x.shape), axis=1)) / 2
-            radiusy = np.nanmedian(np.diff(y.reshape(m.data_specs.y.shape), axis=0)) / 2
+        if len(m._xshape) == 2 and len(m._yshape) == 2:
+            userange = int(np.sqrt(m.set_shape.radius_estimation_range))
+
+            radiusx = (
+                np.nanmedian(
+                    np.diff(x.reshape(m._xshape)[:userange, :userange], axis=1)
+                )
+                / 2
+            )
+            radiusy = (
+                np.nanmedian(
+                    np.diff(y.reshape(m._yshape)[:userange, :userange], axis=0)
+                )
+                / 2
+            )
 
             radius = (radiusx, radiusy)
 
@@ -148,6 +160,11 @@ class shapes(object):
         if radius is None:
             from scipy.spatial import cKDTree
 
+            # take care of 2D data with 1D coordinates
+            if m._1D2D:
+                userange = int(np.sqrt(m.set_shape.radius_estimation_range))
+                x, y = np.meshgrid(x[:userange], y[:userange])
+
             in_tree = cKDTree(
                 np.stack(
                     [
@@ -170,10 +187,11 @@ class shapes(object):
                 radius = (radiusxy, radiusxy)
 
         assert radius is not None, (
-            "EOmaps: Radius estimation failed... either there's something wrong with "
-            + "the provided coordinates or you can try to increase the number of "
-            + "datapoints used to evaluate the radius by increasing "
-            + "`m.set_shape.radius_estimation_range`."
+            "EOmaps: Radius estimation failed... maybe there's something wrong with "
+            "the provided coordinates? "
+            "You can manually specify a radius with 'm.set_shape.<SHAPE>(radius=...)' "
+            "or you can increase the number of datapoints used to estimate the radius "
+            "by increasing `m.set_shape.radius_estimation_range`."
         )
 
         return radius
@@ -184,6 +202,7 @@ class shapes(object):
 
         # special treatment of array input to properly mask values
         array = kwargs.pop("array", None)
+
         if array is not None:
             array = array[mask]
         else:
@@ -239,7 +258,7 @@ class shapes(object):
         def __init__(self, m):
             self._m = m
 
-        def __call__(self, radius, n=20):
+        def __call__(self, radius=1000, n=None):
             """
             Draw geodesic circles with a radius defined in meters.
 
@@ -247,9 +266,10 @@ class shapes(object):
             ----------
             radius : float
                 The radius of the circles in meters.
-            n : int
-                The number of points to calculate on the circle.
-                The default is 20.
+            n : int or None
+                The number of intermediate points to calculate on the geodesic circle.
+                If None, 100 is used for < 10k pixels and 20 otherwise.
+                The default is None.
 
             Returns
             -------
@@ -270,6 +290,20 @@ class shapes(object):
         def _initargs(self):
             return dict(radius=self._radius, n=self.n)
 
+        @property
+        def n(self):
+            if self._n is None:
+                if self._m.data is not None:
+                    if np.size(self._m.data) < 10000:
+                        return 100
+                    else:
+                        return 20
+            return self._n
+
+        @n.setter
+        def n(self, val):
+            self._n = val
+
         @property
         def radius(self):
             return self._radius
@@ -399,7 +433,7 @@ class shapes(object):
 
             coll = PolyCollection(
                 verts,
-                # transOffset=self._m.figure.ax.transData,
+                # transOffset=self._m.ax.transData,
                 **color_and_array,
                 **kwargs,
             )
@@ -411,8 +445,9 @@ class shapes(object):
 
         def __init__(self, m):
             self._m = m
+            self._n = None
 
-        def __call__(self, radius="estimate", radius_crs="in", n=20):
+        def __call__(self, radius="estimate", radius_crs="in", n=None):
             """
             Draw projected ellipses with dimensions defined in units of a given crs.
 
@@ -425,9 +460,10 @@ class shapes(object):
             radius_crs : crs-specification, optional
                 The crs in which the dimensions are defined.
                 The default is "in".
-            n : int
-                The number of points to calculate on the circle.
-                The default is 20.
+            n : int or None
+                The number of intermediate points to calculate on the circle.
+                If None, 100 is used for < 10k pixels and 20 otherwise.
+                The default is None.
             """
             from . import MapsGrid  # do this here to avoid circular imports!
 
@@ -444,6 +480,20 @@ class shapes(object):
         def _initargs(self):
             return dict(radius=self._radius, radius_crs=self.radius_crs, n=self.n)
 
+        @property
+        def n(self):
+            if self._n is None:
+                if self._m.data is not None:
+                    if np.size(self._m.data) < 10000:
+                        return 100
+                    else:
+                        return 20
+            return self._n
+
+        @n.setter
+        def n(self, val):
+            self._n = val
+
         @property
         def radius(self):
             radius = shapes._get_radius(self._m, self._radius, self.radius_crs)
@@ -584,18 +634,19 @@ class shapes(object):
             # mask any point that is in a different quadrant than the center point
             maskx = pts_quadrants != quadrants[:, np.newaxis]
             # take care of points that are on the center line (e.g. don't mask them)
-            # (use a +- 10 degree around 0 as threshold)
+            # (use a +- 25 degree around 0 as threshold)
             cpoints = np.broadcast_to(
-                np.isclose(xp, xc, atol=10)[:, np.newaxis], xs.shape
+                np.isclose(xp, xc, atol=25)[:, np.newaxis], xs.shape
             )
 
             maskx[cpoints] = False
             xs.mask[maskx] = True
-
             ys.mask = xs.mask
 
-            mask = ~np.all(maskx, axis=1) & np.isfinite(theta)
-
+            # mask any datapoint that has less than 4 of the ellipse-points unmasked
+            mask = ~(
+                n - np.count_nonzero(xs.mask, axis=1) <= min(n / 2, 4)
+            ) & np.isfinite(theta)
             return xs, ys, mask
 
         def get_coll(self, x, y, crs, **kwargs):
@@ -619,7 +670,7 @@ class shapes(object):
 
             coll = PolyCollection(
                 verts,
-                # transOffset=self._m.figure.ax.transData,
+                # transOffset=self._m.ax.transData,
                 **color_and_array,
                 **kwargs,
             )
@@ -632,7 +683,7 @@ class shapes(object):
         def __init__(self, m):
             self._m = m
 
-        def __call__(self, radius="estimate", radius_crs="in", mesh=False, n=10):
+        def __call__(self, radius="estimate", radius_crs="in", mesh=False, n=None):
             """
             Draw projected rectangles with fixed dimensions (and possibly curved edges)
 
@@ -653,11 +704,12 @@ class shapes(object):
                 does NOT allow setting edgecolors but it has the advantage that
                 boundaries between neighbouring rectangles are not visible.
                 Only n=1 is currently supported!
-            n : int
-                The number of intermediate points to calculate on the rectangle
-                edges (e.g. to plot "curved" rectangles in projected crs)
-                Use n=1 to get actual rectangles!
-                The default is 10
+            n : int or None
+                The number of intermediate points to calculate on the rectangle edges
+                (e.g. to properly plot "curved" rectangles in projected crs)
+                Use n=1 to force rectangles!
+                If None, 40 is used for <10k datapoints and 10 is used otherwise.
+                The default is None
             """
             from . import MapsGrid  # do this here to avoid circular imports!
 
@@ -669,7 +721,9 @@ class shapes(object):
                 shape.mesh = mesh
 
                 if mesh is True:
-                    if n > 1:
+                    if n is None:
+                        n = 1
+                    elif n > 1:
                         warnings.warn(
                             "EOmaps: rectangles with 'mesh=True' only supports n=1"
                         )
@@ -687,6 +741,26 @@ class shapes(object):
                 mesh=self.mesh,
             )
 
+        @property
+        def n(self):
+            if self._n is None:
+                if self._m.data is not None:
+                    # if plot crs is same as input-crs there is no need for
+                    # intermediate points since the rectangles are not curved!
+                    if self._m._crs_plot == self._m.data_specs.crs:
+                        return 1
+                    elif np.size(self._m.data) < 10000:
+                        return 40
+                    else:
+                        return 10
+                else:
+                    return 10
+            return self._n
+
+        @n.setter
+        def n(self, val):
+            self._n = val
+
         @property
         def radius(self):
             radius = shapes._get_radius(self._m, self._radius, self.radius_crs)
@@ -806,7 +880,7 @@ class shapes(object):
 
             coll = PolyCollection(
                 verts=verts,
-                # transOffset=self._m.figure.ax.transData,
+                # transOffset=self._m.ax.transData,
                 **color_and_array,
                 **kwargs,
             )
@@ -874,7 +948,7 @@ class shapes(object):
 
             coll = TriMesh(
                 tri,
-                # transOffset=self._m.figure.ax.transData,
+                # transOffset=self._m.ax.transData,
                 **color_and_array,
                 **kwargs,
             )
@@ -1000,7 +1074,7 @@ class shapes(object):
             coll = PolyCollection(
                 verts=verts,
                 **color_and_array,
-                # transOffset=self._m.figure.ax.transData,
+                # transOffset=self._m.ax.transData,
                 **kwargs,
             )
 
@@ -1112,33 +1186,29 @@ class shapes(object):
                 maxdist = 4 * np.mean(np.sqrt(radiusx**2 + radiusy**2))
 
                 if radius_crs == "in":
-                    x, y = x[datamask][tri.triangles], y[datamask][tri.triangles]
-                    # get individual triangle side-lengths
-                    l = np.array(
-                        [
-                            np.sqrt(
-                                ((x[:, i] - x[:, j]) ** 2) + ((y[:, i] - y[:, j]) ** 2)
-                            )
-                            for i, j in ((0, 1), (0, 2), (1, 2))
-                        ]
-                    )
+                    # use input-coordinates for evaluating the mask
+                    mx = self._m._props["xorig"].ravel()
+                    my = self._m._props["yorig"].ravel()
                 elif radius_crs == "out":
-                    x0, y0 = x0[datamask][tri.triangles], y0[datamask][tri.triangles]
-
-                    # get individual triangle side-lengths
-                    l = np.array(
-                        [
-                            np.sqrt(
-                                ((x0[:, i] - x0[:, j]) ** 2)
-                                + ((y0[:, i] - y0[:, j]) ** 2)
-                            )
-                            for i, j in ((0, 1), (0, 2), (1, 2))
-                        ]
-                    )
+                    # use projected coordinates for evaluating the mask
+                    mx = x
+                    my = y
                 else:
                     assert (
                         False
                     ), f"the radius_crs '{radius_crs}' is not supported for delaunay-masking"
+
+                mx, my = mx[datamask][tri.triangles], my[datamask][tri.triangles]
+                # get individual triangle side-lengths
+                l = np.array(
+                    [
+                        np.sqrt(
+                            ((mx[:, i] - mx[:, j]) ** 2) + ((my[:, i] - my[:, j]) ** 2)
+                        )
+                        for i, j in ((0, 1), (0, 2), (1, 2))
+                    ]
+                )
+
                 # mask any triangle whose side-length exceeds maxdist
                 mask = np.any(l > maxdist, axis=0)
                 tri.set_mask(mask)
@@ -1173,7 +1243,7 @@ class shapes(object):
 
                 coll = TriMesh(
                     tri,
-                    # transOffset=self._m.figure.ax.transData,
+                    # transOffset=self._m.ax.transData,
                     **color_and_array,
                     **kwargs,
                 )
@@ -1192,7 +1262,7 @@ class shapes(object):
 
                 coll = PolyCollection(
                     verts=verts,
-                    # transOffset=self._m.figure.ax.transData,
+                    # transOffset=self._m.ax.transData,
                     **color_and_array,
                     **kwargs,
                 )
@@ -1233,7 +1303,7 @@ class shapes(object):
                 A callable that takes the image output of the shading pipeline,
                 and returns another Image object.
                 See dynspread() and spread() for examples.
-                The default is `partial(ds.tf.dynspread, max_px=5)`.
+                The default is `partial(ds.tf.dynspread, max_px=50)`.
             agg_hook : callable, optional
                 A callable that takes the computed aggregate as an argument, and returns
                 another aggregate. This can be used to do preprocessing before the
@@ -1252,7 +1322,7 @@ class shapes(object):
                 aggregator = getattr(ds, aggregator)("val")
 
             if shade_hook is None:
-                shade_hook = partial(ds.tf.dynspread, max_px=5)
+                shade_hook = partial(ds.tf.dynspread, max_px=50)
 
             if agg_hook is None:
                 pass
@@ -1394,10 +1464,11 @@ class shapes(object):
         def __init__(self, m):
             self._m = m
             self._radius = None
+            self.radius_crs = "in"
 
-        def __call__(self, radius="estimate", radius_crs="in"):
+        def __call__(self):
             """
-            Draw 2D datasets as rectangles (only 2D datasets, but possibly large ones)
+            Draw the data as a rectangular raster.
 
             (similar to plt.imshow)
 
@@ -1415,7 +1486,7 @@ class shapes(object):
             (e.g. the effective shape is a distorted rectangle with straight edges)
 
             - use `m.set_shape.rectangles()` if you need "curved" edges!
-            - use `m.set_shape.shade_raster()` for very large datasets
+            - use `m.set_shape.shade_raster()` for extremely large datasets
 
             Parameters
             ----------
@@ -1433,16 +1504,11 @@ class shapes(object):
             for m in self._m if isinstance(self._m, MapsGrid) else [self._m]:
                 shape = self.__class__(m)
 
-                shape._radius = radius
-                shape.radius_crs = radius_crs
                 m._shape = shape
 
         @property
         def _initargs(self):
-            return dict(
-                radius=self._radius,
-                radius_crs=self.radius_crs,
-            )
+            return dict()
 
         @property
         def radius(self):
@@ -1507,14 +1573,16 @@ class shapes(object):
             v[:-1, :-1, 0] = p[0] - rx
             v[:-1, :-1, 1] = p[1] - ry
 
-            v[-1, :-1] = v[-2, :-1] + [0, 2 * rx]
-            v[:, -1] = v[:, -2] + [2 * ry, 0]
+            # treat bottom vertices values
+            v[-1, :-1] = v[-2, :-1] + [0, 2 * ry]
+
+            # treat right most vertices values
+            v[:, -1] = v[:, -2] + [2 * rx, 0]
 
             px, py = t.transform(clipx(v[:, :, 0]), clipy(v[:, :, 1]))
             verts = np.stack((px, py), axis=2)
 
             mask = np.logical_and(np.isfinite(px)[:-1, :-1], np.isfinite(py)[:-1, :-1])
-
             return verts, mask
 
         def _get_polygon_coll(self, x, y, crs, **kwargs):
@@ -1527,7 +1595,6 @@ class shapes(object):
 
             # remember masked points
             self._m._data_mask = mask
-
             # don't use a mask here since we need the full 2D array
             color_and_array = shapes._get_colors_and_array(
                 kwargs, np.full_like(mask, True)
@@ -1554,6 +1621,7 @@ class shapes(object):
             return coll
 
         def get_coll(self, x, y, crs, **kwargs):
+
             x, y = np.asanyarray(x), np.asanyarray(y)
             # don't use antialiasing by default since it introduces unwanted
             # transparency for reprojected QuadMeshes!
