@@ -42,7 +42,7 @@ class _WebMap_layer:
 
         styles = list(self.wms_layer.styles)
         if len(styles) == 0:
-            self._style = None
+            self._style = ""
         else:
             self._style = "default" if "default" in styles else styles[0]
 
@@ -82,11 +82,10 @@ class _WebMap_layer:
 
         print(f"\n LEGEND available: {legQ}\n\n" + txt)
 
-    def fetch_legend(self, style=None):
+    def fetch_legend(self, style=None, silent=True):
         if style is None:
             style = self._style
         try:
-
             url = self.wms_layer.styles[style]["legend"]
             legend = requests.get(url)
 
@@ -98,16 +97,18 @@ class _WebMap_layer:
 
                 except ImportError:
                     warn("EOmaps: the legend is '.svg'... please install 'cairosvg'")
+                    return None
             else:
                 img = legend.content
 
             img = Image.open(BytesIO(img))
         except Exception:
-            warn("EOmaps: could not fetch the legend")
-            img = None
+            if not silent:
+                warn("EOmaps: could not fetch the legend")
+            return None
         return img
 
-    def add_legend(self, style=None):
+    def add_legend(self, style=None, img=None):
         """
         Add a legend to the plot (if available)
 
@@ -119,7 +120,8 @@ class _WebMap_layer:
         ----------
         style : str, optional
             The style to use. The default is "default".
-
+        img : BytesIO
+            A pre-fetched legend (if provided the "style" kwarg is ignored!)
         Returns
         -------
         legax : matpltolib.axes
@@ -132,11 +134,20 @@ class _WebMap_layer:
             style = self._style
 
         self._legend_picked = False
+        if img is None:
+            legend = self.fetch_legend()
+        else:
+            legend = img
 
-        legend = self.fetch_legend()
         if legend is not None:
-            axpos = self._m.figure.ax.get_position()
-            legax = self._m.figure.f.add_axes((axpos.x0, axpos.y0, 0.25, 0.5))
+            if not hasattr(self, "_layer"):
+                # use the currently active layer if the webmap service has not yet
+                # been added to the map
+                print("EOmaps: The WebMap for the legend is not yet added to the map!")
+                self._layer = self._m.BM._bg_layer
+
+            axpos = self._m.ax.get_position()
+            legax = self._m.f.add_axes((axpos.x0, axpos.y0, 0.25, 0.5))
 
             legax.patch.set_visible(False)
             legax.tick_params(
@@ -146,7 +157,11 @@ class _WebMap_layer:
             legax.set_aspect(1, anchor="SW")
             legax.imshow(legend)
 
-            self._m.BM.add_artist(legax)
+            # hide the legend if the corresponding layer is not active at the moment
+            if self._layer not in self._m.BM._bg_layer.split("|"):
+                legax.set_visible(False)
+
+            self._m.BM.add_artist(legax, self._layer)
 
             def cb_move(event):
                 if not self._legend_picked:
@@ -154,9 +169,9 @@ class _WebMap_layer:
 
                 # only execute action if no toolbar action is active
                 if (
-                    hasattr(self._m.figure.f.canvas, "toolbar")
-                    and self._m.figure.f.canvas.toolbar is not None
-                    and self._m.figure.f.canvas.toolbar.mode != ""
+                    hasattr(self._m.f.canvas, "toolbar")
+                    and self._m.f.canvas.toolbar is not None
+                    and self._m.f.canvas.toolbar.mode != ""
                 ):
                     return
 
@@ -171,7 +186,7 @@ class _WebMap_layer:
                     legax.bbox.height,
                 )
 
-                bbox = bbox.transformed(self._m.figure.f.transFigure.inverted())
+                bbox = bbox.transformed(self._m.f.transFigure.inverted())
                 legax.set_position(bbox)
 
             def cb_release(event):
@@ -186,6 +201,8 @@ class _WebMap_layer:
                     legax.set_frame_on(False)
                     self._legend_picked = False
 
+            # TODO add keypress callback to remove legend!
+
             def cb_scroll(event):
                 if not self._legend_picked:
                     return
@@ -204,16 +221,11 @@ class _WebMap_layer:
 
                 self._m.BM.update()
 
-            self._m.figure.f.canvas.mpl_connect("scroll_event", cb_scroll)
-            self._m.figure.f.canvas.mpl_connect("button_press_event", cb_pick)
-            self._m.figure.f.canvas.mpl_connect("button_release_event", cb_release)
-            self._m.figure.f.canvas.mpl_connect("motion_notify_event", cb_move)
+            self._m.f.canvas.mpl_connect("scroll_event", cb_scroll)
+            self._m.f.canvas.mpl_connect("button_press_event", cb_pick)
+            self._m.f.canvas.mpl_connect("button_release_event", cb_release)
+            self._m.f.canvas.mpl_connect("motion_notify_event", cb_move)
 
-            if not hasattr(self, "_layer"):
-                # use the currently active layer if the webmap service has not yet
-                # been added to the map
-                print("EOmaps: The WebMap for the legend is not yet added to the map!")
-                self._layer = self._m.BM._bg_layer
             self._m.parent._wms_legend.setdefault(self._layer, list()).append(legax)
 
             self._m.BM.update()
@@ -284,8 +296,8 @@ class _WebMap_layer:
 
         (x0, x1), (y0, y1) = transformer.transform((x0, x1), (y0, y1))
 
-        self._m.figure.ax.set_xlim(x0, x1)
-        self._m.figure.ax.set_ylim(y0, y1)
+        self._m.ax.set_xlim(x0, x1)
+        self._m.ax.set_ylim(y0, y1)
 
     def _set_style(self, style=None):
         # style is a list with 1 entry!
@@ -354,7 +366,13 @@ class _wmts_layer(_WebMap_layer):
 
             if self._layer == "all" or self._m.BM.bg_layer == self._layer:
                 # add the layer immediately if the layer is already active
-                self._do_add_layer(self._m, self._layer)
+                self._do_add_layer(
+                    self._m,
+                    l=self._layer,
+                    wms_kwargs=kwargs,
+                    zorder=zorder,
+                    alpha=alpha,
+                )
             else:
                 if self._layer not in self._m._get_layers():
                     # create a new (empty) layer so that utility-widgets get updated!
@@ -378,9 +396,9 @@ class _wmts_layer(_WebMap_layer):
     # for SlippyImageArtist
     @staticmethod
     def _add_wmts(ax, wms, layers, wms_kwargs=None, **kwargs):
-        from cartopy.io.ogc_clients import WMSRasterSource
+        from cartopy.io.ogc_clients import WMTSRasterSource
 
-        wms = WMSRasterSource(wms, layers, getmap_extra_kwargs=wms_kwargs)
+        wms = WMTSRasterSource(wms, layers, gettile_extra_kwargs=wms_kwargs)
 
         # Allow a fail-fast error if the raster source cannot provide
         # images in the current projection.
@@ -399,13 +417,6 @@ class _wmts_layer(_WebMap_layer):
             m.ax, self._wms, self.name, interpolation="spline36", **kwargs
         )
 
-        # art = m.figure.ax.add_wmts(
-        #     self._wms,
-        #     self.name,
-        #     wmts_kwargs=kwargs,
-        #     interpolation="spline36",
-        #     zorder=zorder,
-        # )
         m.BM.add_bg_artist(art, l)
 
 
@@ -461,7 +472,13 @@ class _wms_layer(_WebMap_layer):
 
             if self._layer == "all" or m.BM.bg_layer == self._layer:
                 # add the layer immediately if the layer is already active
-                self._do_add_layer(m, self._layer)
+                self._do_add_layer(
+                    m=m,
+                    l=self._layer,
+                    wms_kwargs=kwargs,
+                    zorder=zorder,
+                    alpha=alpha,
+                )
             else:
                 if self._layer not in self._m._get_layers():
                     # create a new (empty) layer so that utility-widgets get updated!
@@ -1147,6 +1164,7 @@ class SlippyImageArtist_NEW(AxesImage):
 
         ax.callbacks.connect("xlim_changed", self.on_xlim)
         self._prev_extent = (0, 0)
+        self._prev_size = (ax.bbox.width, ax.bbox.height)
 
     def on_xlim(self, *args, **kwargs):
         self.stale = True
@@ -1164,7 +1182,16 @@ class SlippyImageArtist_NEW(AxesImage):
             window_extent = ax.get_window_extent()
             [x1, y1], [x2, y2] = ax.viewLim.get_points()
 
-            if self._prev_extent != (x1, x2, y1, y2) or len(self.cache) == 0:
+            # only fetch images if one of the following is true:
+            # - the map extent changed
+            # - the size of the axis changed
+            # - the cache is empty
+
+            if (
+                self._prev_extent != (x1, x2, y1, y2)
+                or self._prev_size != (ax.bbox.width, ax.bbox.height)
+                or len(self.cache) == 0
+            ):
                 # only re-fetch tiles if the extent has changed
                 located_images = self.raster_source.fetch_raster(
                     ax.projection,
@@ -1173,17 +1200,19 @@ class SlippyImageArtist_NEW(AxesImage):
                 )
                 self.cache = located_images
                 self._prev_extent = (x1, x2, y1, y2)
+                self._prev_size = (ax.bbox.width, ax.bbox.height)
 
             for img, extent in self.cache:
                 self.set_array(img)
                 with ax.hold_limits():
                     self.set_extent(extent)
-                super().draw(renderer, *args, **kwargs)
+
+            super().draw(renderer, *args, **kwargs)
 
             self.stale = False
+
         except Exception:
             print("EOmaps: ... could not fetch WebMap service")
-
             if self in self.axes._mouseover_set:
                 self.axes._mouseover_set.remove(self)
 
